/**
 * 
 */
package com.asseco.ufe.platform.widgets;

import java.util.Arrays;

import org.eclipse.rwt.lifecycle.WidgetUtil;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;

import com.asseco.ufe.platform.dictionary.api.DictionaryItemTO;
import com.asseco.ufe.platform.dictionary.api.DictionaryTO;
import com.asseco.ufe.platform.widgets.events.DictionaryItemEvent;
import com.asseco.ufe.platform.widgets.events.DictionaryItemListener;
import com.asseco.ufe.platform.widgets.internal.Messages;

/**
 * Kontrolka udostpnia moliwo wyboru wartoci z listy rozwijalnej, jest
 * zintegrowana z klasami obsugujcymi sowniki.
 * W pierwszym wierszu wywietla pozycj "Brak wartoci" a w kolejnych pozycje n-
 * tego poziomu sownika.
 * 
 * Przykady uycia
 * <ul>
 * 	<li>poniszy kod powoduje wyswietlenie pierwszego poziomu sownika w
 * licie</li>
 * </ul>
 * DictionaryTO myDictionary = dictionaryService.getDictionary(MY_DICTIONARY_CODE,
 * locale);
 * dictionaryBox.setRoot(myDictionary);
 * 
 * <ul>
 * 	<li>poniszy kod powoduje wywietlenie drugiego poziomu sownika w licie:
 * </li>
 * </ul>
 * DictionaryItemTO item = myDictionary.getItems()[0];
 * dictionaryBox.setRoot(item);
 * 
 * <ul>
 * 	<li>poniszy kod zwraca wybran pozycj sownikow:</li>
 * </ul>
 * DictionaryItem selected = dictionaryBox.getSelectedItem();
 */


public class DictionaryBox extends AbstractBox<DictionaryItemTO> {

	protected java.util.List<DictionaryItemTO>  items;
	public java.util.List<DictionaryItemListener> listeners;
	private Text codeText;
	private Text descriptionText;
	protected CCombo combo;


	public DictionaryBox(Composite parent, int style) {
		super(parent, style);
		listeners = new java.util.ArrayList<DictionaryItemListener>();
	}

	// TODO do usuniecia
	public DictionaryBox setText(String code, String description) {
		codeText.setText(code);
		descriptionText.setText(description);
		return this;
	}


	/**
	 * Dodaje listenera do listy listenerw
	 */
	public void addDictionaryItemListener(DictionaryItemListener listener) {
		this.listeners.add(listener);
	}

	/**
   * Usuwa listenera z listy listenerw
   */
	public void removeDictionaryItemListener(DictionaryItemListener listener){
		this.listeners.remove(listener);
	}
	
	/**
	 * 
	 * @return items
	 * <br>Zwraca tablic elementw sownika, ktre s wywietlane w polu wyboru.
	 */
	public DictionaryItemTO[] getItems() {
		return (DictionaryItemTO[])items.toArray();
	}
	
	/**
	 * Wywoanie funkcji spowoduje, e na licie zostanie wybrana dana pozycja
	 * sownikowa.
	 * @throws IllegalArgumentException jeli <i>item</i> nie zawiera si w licie
	 * pozycji sownikowych przekazanych w wywoaniu <i>setItems</i>
	 * 
	 * @param item
	 */
	public void selectItem(DictionaryItemTO item){
    if (item == null)
      throw new IllegalArgumentException("item cannot be null");
    int index = items.indexOf(item);
    if (index >= 0)
      combo.select(index+1);
    else
      throw new IllegalArgumentException("item is not in the dictionary");
  }

	
	
	/**
	 * Wywoanie funkcji spowoduje, e na licie zostanie pozycja sownikowa o podanym
	 * kodzie
	 * @throws IllegalArgumentException jeli <i>itemCode</i> nie jest kodem nalecym
	 * do listy pozycji sownikowych przekazanych w wywoaniu <i>setItems</i>
	 * 
	 * @param itemCode
	 */
	public void selectItem(String itemCode){
	  if(itemCode == null){
	    combo.select(0);
	    return;
	  }
    DictionaryItemTO item = findByCode(itemCode);

    try{
    	selectItem(item);
    }
    catch(IllegalArgumentException excI){
    	//TODO - koniecznie do poprawy
    	//jak nie ma to wyswietlam brak wartosci. Docelowo prawdopodobnie bedzie tu informacja e nie ma wartosci w slwoniku na bazie danych lub nie ma caego slwonika
    	//ponadto chyba lepiej zucic swoj wyjatek z jakims kodem zmiast equals po opisie
    	combo.select(0);
    	if(excI.getMessage().equals("item cannot be null")){
    		
    	}
    	else{
    		if(excI.getMessage().equals("item is not in the dictionary")){
    			
    		}
    	}
    }
    
	}
	
	/**
	 * Sets the number of items that are visible in the drop down portion of the
	 * receiver's list.
	 * 
	 * @param n
	 */
	public void setVisibleItemCount(int n){
		combo.setVisibleItemCount(n);

	}


	/**
	 * @return {@link DictionaryItemTO}
	 */
	public DictionaryItemTO getSelectedItem() {
		return getValue();
	}
	
	/**
	 * Buduje list pozycji dla
	 * @param items {@link DictionaryItemTO}
	 */
	public void setItems(DictionaryItemTO[] items) {

		this.items = Arrays.asList(items);

	}

	/**
	 * @return true jeli nie wybrano pozycji
	 */
	public boolean isEmpty() {
		int selectionIndex = combo.getSelectionIndex();
		return (selectionIndex>0) ? false: true;
	}

	@Override
	public Composite create(Composite parent) {

		Composite composite = new Composite(this, SWT.NONE);
		FillLayout fillLayout = new FillLayout(SWT.HORIZONTAL);
//		fillLayout.marginWidth=2;
//		fillLayout.marginHeight=2;
		composite.setLayout(fillLayout);
		combo = new CCombo(composite, SWT.READ_ONLY);
		
		combo.addSelectionListener(new SelectionListener() {
      
      @Override
      public void widgetSelected(SelectionEvent e) {
        fireItemSelected();
        }

      @Override
      public void widgetDefaultSelected(SelectionEvent e) {
       fireItemSelected();
      }
		});

		return composite;
	}

	@Override
	public void clear() {
		combo.select(0);
	}

	@Override
	public DictionaryItemTO getValue() {
		int index = combo.getSelectionIndex();
		if (index<=0)
			return null;
		else
		  return findByValue(combo.getItem(index));
	}

	@Override
	public void setValue(DictionaryItemTO value) {
	  selectItem(value);

	}
	
  @Override
  public void setEnabled(boolean enabled){
    super.setEnabled(enabled);
    combo.setEnabled(enabled);
  }

	/**Funkcja buduje list pozycji sownikowych na podstawie przekazanego sownika. Zostan wywietlone pozycje pierwszego poziomu sownika.
	 * @param dictionaryTo Slownik
	 */
	public void setRoot(DictionaryTO dictionaryTo) {
		setItems(dictionaryTo.getItems());
		fillDictionaryBox();
	}

	public void	setRoot(DictionaryItemTO dicItemTO) {
		//TODO wypelnienie ementow dzieci
		setItems(dicItemTO.getChildren());
		fillDictionaryBox();
		
	}

	private void fillDictionaryBox() {
		combo.removeAll();
		combo.add(Messages.get("DictionaryBox.NoValue"));
		for (DictionaryItemTO dicItem : items) {
			combo.add(dicItem.getValue());
		}
		combo.select(0);
	}
	
	protected DictionaryItemTO findByCode(String code){
    DictionaryItemTO result=null;
    if (code == null) return result;
   
    for (DictionaryItemTO element : items) {
      if (element.getCode().equals(code)){
        result = element;
        break;
      }
        
    }
    return result;
  }
	protected DictionaryItemTO findByValue(String value){
    DictionaryItemTO result=null;
    if (value == null) return result;
   
    for (DictionaryItemTO element : items) {
      if (element.getValue().equals(value)){
        result = element;
        break;
      }
        
    }
    return result;
  }
	protected void fireItemSelected () {
    for (DictionaryItemListener listener : listeners) {
      DictionaryItemEvent dictionaryItemEvent = new DictionaryItemEvent();
      dictionaryItemEvent.item = getSelectedItem();
      listener.itemSelected(dictionaryItemEvent);
    }
    
  }
}


